"""
	Features :
		- data collection
		- detection, reboot	(service)
		- respond to commands	(service)
"""
#from threading import Thread, RLock
from __future__ import division
import os
import json
import time
from datetime import datetime
from utils.transport import *
import monitor
import requests
from threading import Thread
class Manager() :
	def version(self):
		return 1.1
	"""
		
		delay : <value>
		limit : <value>
		scope : apps,folders,learner,sandbox
	"""
	def __init__(self):
		self.factory	= DataSourceFactory()
	def set(self,name,value):
		setattr(name,value)
	def init(self,**args) :
		self.id		= args['node']
		self.agents 	= args['agents']
		self.config	= dict(args['config'])
		self.key	= args['key']
		self.actors	= args['actors']
		self.plan	= self.config['plan']
		
		self.DELAY  = int(self.plan['metadata']['delay'])
		self.host = args['host']
		self.update()	#-- Initializing status information
		_args={"host":"dev.the-phi.com","qid":self.id,"uid":self.key}
		#
		# Connecting to the messaging service
		
		self.qlistener = self.factory.instance(type="QueueListener",args=_args)
		self.qlistener.callback = self.callback
		self.qlistener.init(self.id)
		
		# self.qlistener.read()
		thread = (Thread(target=self.qlistener.read))
		thread.start()
	
	def update(self) :
		"""
			This method inspect the plans for the current account and makes sure it can/should proceed
			The user must be subscribed and to the service otherwise this is not going to work
		"""
		
		url="http://:host/monitor/init/collector".replace(':host',self.host)
			
		r = requests.post(url,headers={"key":self.key,"id":self.id})
		r =  json.loads(r.text)
		# meta =  [item['metadata'] for item in plans if item['status']=='active' ]
		self.plan = r['plan']
		meta = self.plan['metadata']
		
		if meta :
			self.DELAY = 60* int(meta['delay'])
			self.LIMIT = int(meta['limit'])
			#dbname = [item['name'] for item in plans if int(item['metadata']['limit']) == self.LIMIT][0]
			#self.config['store']['args']['dbname'] = dbname

		else:
			self.DELAY = -1
			self.LIMIT = -1
	
		#self.filter(meta)
		#
		# We are removing all that is not necessary i.e making sure the features matches the plan user has paid for
		#
		self.agents  = self.filter('agents',meta,self.agents)
		self.actors = self.filter('actors',meta,self.actors)
		self.setup(meta)

	# def filter_collectors(self,meta) :
	# 	"""
	# 		remove collectors that are not specified by the plan
	# 		Note that the agents (collectors) have already been initialized ?
	# 	"""
	# 	values = meta['agents'].replace(' ','').split(',')
	# 	self.agents = [agent for agent in self.agents if agent.getName() in values]

		
	# def filter_actors(self,meta):
	# 	"""
	# 		removes actors that are NOT specified by the subscription plan
	# 		Note that the actor have already been instatiated and need initialization
	# 	"""
	# 	values = meta['actors'].replace(' ','').split('.')
	# 	self.actors = [actor for actor in self.actors if actor.getName() in values]
	
	def filter(self,id,meta,objects):
		"""
			This function filters the agents/actors given what is available in the user's plan
			
		"""
		values = meta[id].replace(' ','').split(',')
		return [item for item in objects if item.getName() in values]
	
	def setup(self,meta) :
		# conf = {"folders":None,"apps":None}
		# read_class 	= self.config['store']['class']['read']
		# read_args	= self.config['store']['args']
		

		# args = None
		# couchdb	= self.factory.instance(type=read_class,args=read_args)
		# args 	= couchdb.view('config/apps',key=self.key)
		# if len(args.keys()) > 0 :
		# 	self.apply_setup('apps',args)
		# args = couchdb.view('config/folders',key=self.key)
		# if 'folder_size' not in meta :
		# # 	args['threshold'] = meta['folder_size']
		# 	self.apply_setup('folders',meta)			
		#self.apply_setup('folders',meta)
		#@TODO: For now app actors don't need any particular initialization
		pass
			
	def apply_setup(self,name,args) :
		for actor in self.actors :
			if args is not None and actor.getName() == name and len(args.keys()) > 0:												
				actor.init(args)
		
	def isvalid(self):
		self.update()
		return self.DELAY > -1 and self.LIMIT > -1
	def post(self,row) :
		"""
			This function is designed to take appropriate action if a particular incident has been detected
			@param label	
			@param row	data pulled extracted
		"""
		message = {}
		message['action'] = 'reboot'
		message['node']	= label

	def callback(self,channel,method,header,stream):
		"""
			This function enables the manager to be able to receive messages and delegate them to the appropriate actor
			@channel
		"""
		message = json.loads(stream)
		#
		# we should inspect the message and insure it has reached the appropriate recepient
		#
		if 'node' in message and message['node'] == self.id :
			action = message['action']
			params = message['params']
			# params['plan'] = self.plan['metadata']
			self.delegate(action,params)

	def delegate(self,action,params):
		for actor in self.actors :
			
			if actor.isValid(action=action,params=params) :
				
				actor.init(action,params)
				actor.run()

				break
		pass
	def run(self):
		#DELAY=35*60 #- 35 Minutes
		#LIMIT=1000
		COUNT = 0
		COUNT_STOP 	= int(24*60/ self.DELAY)
		write_class	= self.config['store']['class']['write']
		read_args	= self.config['store']['args']
		
		while True :
			COUNT += 1
			if COUNT > COUNT_STOP :
				if self.isvalid() :
					COUNT = 0
				else:
					break
			for agent in self.agents :
				data	= agent.composite()
				label	= agent.getName()
				node	= '@'.join([label,self.id])
				row		= {}
				
				if label == 'folders':
					row = [ dict({"id":self.id}, **_row) for _row in data]										
				else:
					#label = id
					row = data
					#
					# @TODO:
					# This data should be marked if it has been flagged for reboot
					# 
				if type(row)==list and len(row) == 0 :

					continue
				#
				#
				index = self.agents.index(agent)

				if len(self.actors) > index and self.actors[index].getName() == agent.getName() :
					actor = self.actors[index]
					actor.analyze(row)

				# self.lock.acquire()
				store = self.factory.instance(type=write_class,args=read_args)
				store.flush(size=self.LIMIT)				
				store.write(label=node,row=[row])
				# self.lock.release()
			print (["Falling asleep ",self.DELAY/60])
			time.sleep(self.DELAY)
				
